/*
 *  Roadnav
 *  CyborgNavigator.h
 *
 *  Copyright (C) 2008 Dustin Spicuzza <dustin@virtualroadside.com>
 *
 *  Optimized port of python implementation originally written by 
 *  Randall Munroe (xkcd.com), 2008-05-17
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of version 2 of the GNU General Public License
 *  as published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */


//////////////////////////////////////////////////////////////////////////////
/// \file
///
/// This is a class for reading aloud cardinal directions to a specified 
/// destination
///
//////////////////////////////////////////////////////////////////////////////

#ifndef CYBORGNAVIGATOR_H
#define CYBORGNAVIGATOR_H

#include <wx/config.h>

#include "TTS.h"
#include <libroadnav/MapMath.h>
#include <libroadnav/MapRepresentations.h>

#include "wxGPSEvent.h"

class CyborgNavigator {
	public:

		CyborgNavigator(double lat, double lon);
		void ReloadSettings();

		// call this to set a new target
		void SetDestination(double lat, double lon);

		// call this once a second, or something to that effect
		void Navigate(const wxGPSEvent &evGPS, ITTS * m_tts);

	private:

		void Vectorize(double x1, double y1, double x2, double y2, double &x, double &y);
		double Angle(double x, double y);
		double GetOffset(double heading, double angle);
		int Offset2OClock(double angle);
		wxString& GetDirectionalWord(double angle);

		// use these to calculate average speed
		void CalculateAverageSpeed(double speed);
		double m_avgSpeed_total;
		double m_avgSpeed_last;
		int m_avgSpeed_n;

		// list of direction words
		wxArrayString m_words;

		double m_destination_lat;
		double m_destination_lon;
		double m_last_heading;

		// what units should we use?
		EUnits m_units;

		// timeouts and such
		time_t m_last_distance_speech, m_distance_timeout;
		time_t m_last_direction_change_speech;
		time_t m_last_direction_speech, m_direction_timeout;
		time_t m_last_gps_fix_speech, m_gps_fix_speech_timeout;

};

#endif
