/*
 *  Roadnav
 *  NavTypeDialog.cpp
 *
 *  Copyright (c) 2008 Dustin Spicuzza <dustin@virtualroadside.com>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of version 2 of the GNU General Public License
 *  as published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

//////////////////////////////////////////////////////////////////////////////
/// \file
///
/// This file contains the NavTypeDialog, a simple dialog to ask the user
/// what type of navigation they want to use
///
//////////////////////////////////////////////////////////////////////////////


#include <wx/wx.h>
#include <wx/sizer.h>

#include "NavTypeDialog.h"


BEGIN_EVENT_TABLE(NavTypeDialog, wxDialog)
	EVT_BUTTON(navTypeTraditional, NavTypeDialog::OnSelect)
	EVT_BUTTON(navTypeCyborg, NavTypeDialog::OnSelect)
	EVT_BUTTON(wxID_CANCEL, NavTypeDialog::OnSelect)
END_EVENT_TABLE()



NavTypeDialog::NavTypeDialog(wxWindow *parent, wxFont * pFont) : wxDialog(parent, -1, wxString(wxT("Select Navigation Type")))
{
	if (pFont)
		SetFont(*pFont);

	wxBoxSizer * topSizer = new wxBoxSizer(wxHORIZONTAL);

	// add buttons
	topSizer->Add(
		new wxButton(this, navTypeTraditional, _("Traditional")),
		wxSizerFlags().Border(wxALL,10).Left()
	);

	topSizer->Add(
		new wxButton(this, navTypeCyborg, _("Cyborg")),
		wxSizerFlags().Border(wxALL,10).Right()
	);

	// add a cancel button
	wxSizer * szr = CreateButtonSizer(wxCANCEL);

	if (szr)
	{
		wxBoxSizer * sizer = new wxBoxSizer(wxVERTICAL);
		sizer->Add(topSizer, wxSizerFlags().Border(wxALL,10));
		sizer->Add(szr, wxSizerFlags().Border(wxALL,10));
		topSizer = sizer;
	}

	SetSizer(topSizer);
	topSizer->Fit(this);	
}

void NavTypeDialog::OnSelect(wxCommandEvent &event)
{
	EndModal(event.GetId());
}
