/*
 *  Roadnav
 *  OptionCustom.cpp 
 *
 *  Copyright (c) 2008 Dustin Spicuzza <dustin@virtualroadside.com>
 *  Copyright (c) 2004 - 2008 Richard L. Lynch <rllynch@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of version 2 of the GNU General Public License
 *  as published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
//////////////////////////////////////////////////////////////////////////////
/// \file
///
/// Implementation of custom options for special cases. It may make sense 
/// to break the bigger options out into their own files
///
//////////////////////////////////////////////////////////////////////////////

#include "wxOptionGroup.h"
#include "OptionCustom.h"

#include "TTSFactory.h"
#include "wxSkinnableFrame.h"

void serialPortComboBoxOption::DoDisplay(wxOptionSizer * sizer, wxWindow * parent){
	wxString port;
	int i = 0;

	// enumerate serial ports and add them to our list -- we do this each time we're displayed
	// so that we can reflect anything thats been added/removed to system... since we don't
	// have any kind of device notification mechanism 

	m_items.Clear();
	m_comboBox = NULL;

	while (true) {
		if ((port = EnumSerialPort(i++)) == wxT(""))
			break;
		Add(port);
	}
	
	// call the base function to do the actual work
	this->wxComboBoxOption::DoDisplay(sizer, parent);
}

ttsTypeOption::ttsTypeOption(const wxString& configValueName, const wxString& description, const wxString& defaultValue) :
	wxComboBoxOption(configValueName, description, defaultValue, true)
{	
	for (int iTTSType = FIRST_TTSTYPE; iTTSType <= LAST_TTSTYPE; iTTSType++)
	{
		ETTSType eTTSType = (ETTSType) iTTSType;
		ITTS * pcTTS;
		
		pcTTS = ConstructTTS(eTTSType);
		
		if (eTTSType == TTSTypeNone || pcTTS->Name() != wxT("None"))
			Add(iTTSType, pcTTS->Name());
		
		delete pcTTS;
	}
}

void ttsTypeOption::DoDisplay(wxOptionSizer * sizer, wxWindow * parent)
{
	this->wxComboBoxOption::DoDisplay(sizer, parent);

	// add more stuff
	sizer->Add(m_testButton = new wxButton(parent, wxID_ANY, _("Test TTS")),0, wxALIGN_RIGHT);
	sizer->Add(m_testText = new wxTextCtrl(parent, wxID_ANY, _("Test phrase"), wxDefaultPosition, wxSize(200,-1)));

	// pass a pointer to the this pointer... see wxBoolOption::Display
	m_testButton->Connect(m_testButton->GetId(), wxEVT_COMMAND_BUTTON_CLICKED, 
		wxCommandEventHandler(ttsTypeOption::OnTest), wxDynamicCast(new wxOptionPtr(this),wxObject));
}



void ttsTypeOption::OnTest(wxCommandEvent& event)
{
	// see wxBoolOption::Display
	ttsTypeOption * _this = (ttsTypeOption*)wxDynamicCast ( event.m_callbackUserData, wxOptionPtr )->option;
	size_t i = 0;

	// set i to index the indexed TTS type
	for (i = 0; i < _this->m_items.Count(); i++)
		if (_this->m_comboBox->GetValue() == _this->m_items[i])
			break;

	// this shouldn't happen
	if (i == _this->m_items.Count())
		return;

	ITTS * pcTTS = ConstructTTS((ETTSType)_this->m_indexes[i]);
	
	if (pcTTS->Speak(_this->m_testText->GetValue()) != ITTS::TTSStatusOK)
	{
		::wxMessageBox(pcTTS->GetLastError(),
						_("Error"),
						wxOK | wxICON_ERROR);
	}
	
	delete pcTTS;
}

skinComboBoxOption::skinComboBoxOption(const wxString& configValueName, const wxString& description, const wxString& defaultValue) :
	wxComboBoxOption(configValueName, description, defaultValue, false)
{
	wxArrayString skins = EnumerateSkins();
	for (size_t i = 0; i < skins.Count(); i++)
		Add(skins[i]);
}

