/*
 *  Roadnav
 *  mapThemeOption.cpp
 *
 *  Copyright (c) 2004 - 2008 Richard L. Lynch <rllynch@users.sourceforge.net>
 *  Copyright (c) 2008 Dustin Spicuzza <dustin@virtualroadside.com>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of version 2 of the GNU General Public License
 *  as published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

//////////////////////////////////////////////////////////////////////////////
/// \file
///
/// Contains the map themes option
///
//////////////////////////////////////////////////////////////////////////////

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#ifdef _MSC_VER
#pragma warning(disable: 4786)
#pragma warning(disable: 4800)
#endif

#include <wx/wx.h>

#include "libroadnav/Debug.h"

#include "LCDprocInterface.h"
#include "wxOptions.h"
#include "OptionThemes.h"
#include "ThemeManager.h"

#include "EditThemeDialog.h"


//////////////////////////////////////////////////////////////////////////////
///
/// mapThemeOption constructor
///
//////////////////////////////////////////////////////////////////////////////
mapThemeOption::mapThemeOption(IMapData * pMapData, ThemeManager * pThemeManager)  :
    wxOption(wxT(""), wxT("")), 
        m_pMapData(pMapData), 
        m_pParentThemeManager(pThemeManager)
{ 
	m_ThemeManager = *m_pParentThemeManager;

    m_nightTheme = m_ThemeManager.GetNightThemeName();
    m_dayTheme = m_ThemeManager.GetDayThemeName();
    m_photoTheme = m_ThemeManager.GetPhotoThemeName();

    m_themeUsage = m_ThemeManager.GetThemeUsage();
}


void mapThemeOption::DoDisplay(wxOptionSizer * sizer, wxWindow * parent)
{
    wxFlexGridSizer* psizerGrid = new wxFlexGridSizer(1, 3, 0, 0);
	
	//////////////////////////////////////////////////////////////////////////
	// List of themes
	//////////////////////////////////////////////////////////////////////////
    wxBoxSizer* psizerThemeList = new wxBoxSizer(wxVERTICAL);
	psizerThemeList->Add( new wxStaticText(parent, -1, wxT("Manage Themes")), 0, wxALL, 5 );
    m_pctlThemeList = new wxListBox(parent, wxID_ANY, wxDefaultPosition, wxDefaultSize, 0, NULL, wxLB_SINGLE|wxLB_SORT);
	psizerThemeList->Add(m_pctlThemeList, 1, wxALL|wxEXPAND, 5);
	psizerGrid->Add(psizerThemeList, 1, wxEXPAND, 0);

	//////////////////////////////////////////////////////////////////////////
	// Command buttons
	//////////////////////////////////////////////////////////////////////////
	wxBoxSizer* psizerCommandButtons = new wxBoxSizer(wxVERTICAL);
    wxButton* tmp;

	psizerCommandButtons->Add(20,20,0,wxALL,5);
    
    psizerCommandButtons->Add( tmp = new wxButton(parent, wxID_ANY, wxT("Add...")), 0, wxALL, 5);
    tmp->Connect(tmp->GetId(), wxEVT_COMMAND_BUTTON_CLICKED, 
		wxCommandEventHandler(mapThemeOption::OnAdd), wxDynamicCast(new wxOptionPtr(this),wxObject));

    psizerCommandButtons->Add( tmp = new wxButton(parent, wxID_ANY, wxT("Edit...")), 0, wxALL, 5);
    tmp->Connect(tmp->GetId(), wxEVT_COMMAND_BUTTON_CLICKED, 
		wxCommandEventHandler(mapThemeOption::OnEdit), wxDynamicCast(new wxOptionPtr(this),wxObject));
    
    psizerCommandButtons->Add( tmp = new wxButton(parent, wxID_ANY, wxT("Remove")), 0, wxALL, 5);
    tmp->Connect(tmp->GetId(), wxEVT_COMMAND_BUTTON_CLICKED, 
		wxCommandEventHandler(mapThemeOption::OnRemove), wxDynamicCast(new wxOptionPtr(this),wxObject));
	
    psizerGrid->Add(psizerCommandButtons, 1, wxEXPAND, 0);

	//////////////////////////////////////////////////////////////////////////
	// Active themes
	//////////////////////////////////////////////////////////////////////////
	wxString strings(wxT("Default"));
	wxBoxSizer* psizerActiveThemes = new wxBoxSizer(wxVERTICAL);
	psizerActiveThemes->Add( new wxStaticText(parent,-1,wxT("Nighttime Theme")), 0, wxALL, 5);
	m_pctlNightTheme = new wxComboBox(parent,wxID_ANY,wxT(""),wxDefaultPosition,wxDefaultSize,0,NULL,wxCB_READONLY);
	psizerActiveThemes->Add( m_pctlNightTheme, 1, wxALL | wxEXPAND, 5);

	psizerActiveThemes->Add( new wxStaticText(parent,-1,wxT("Daytime Theme")), 0, wxALL, 5);
	m_pctlDayTheme = new wxComboBox(parent,wxID_ANY,wxT(""),wxDefaultPosition,wxDefaultSize,0,NULL,wxCB_READONLY);
	psizerActiveThemes->Add( m_pctlDayTheme, 1, wxALL | wxEXPAND, 5);
	
	psizerActiveThemes->Add( new wxStaticText(parent,-1,wxT("Aerial Photo Theme")), 0, wxALL, 5);	
	m_pctlPhotoTheme = new wxComboBox(parent,wxID_ANY,wxT(""),wxDefaultPosition,wxDefaultSize,0,NULL,wxCB_READONLY);
	psizerActiveThemes->Add( m_pctlPhotoTheme, 1, wxALL | wxEXPAND, 5);

	wxString strChoices[] = {wxString(wxT("Swap at sunrise/sunset")),wxString(wxT("Nighttime Only")),wxString(wxT("Daytime Only"))};
	m_pctlThemeUsage = new wxRadioBox(parent,wxID_ANY,wxT("Theme Usage"),wxDefaultPosition,wxDefaultSize,3,strChoices,1,wxRA_SPECIFY_COLS);
	psizerActiveThemes->Add( m_pctlThemeUsage, 0, wxALL, 5);

	psizerGrid->Add(psizerActiveThemes, 1, wxEXPAND, 0);
	
	//////////////////////////////////////////////////////////////////////////
	// Set up the sizer
	//////////////////////////////////////////////////////////////////////////

    sizer->Add(psizerGrid, wxOptionSizerFlags().ColSpan(2));

    //////////////////////////////////////////////////////////////////////////
	// Fill up the theme lists
	//////////////////////////////////////////////////////////////////////////
	wxArrayString strThemes = m_ThemeManager.GetThemeNames();
	
	m_pctlThemeList->Append(strThemes);
	m_pctlNightTheme->Append(strThemes);
	m_pctlDayTheme->Append(strThemes);
	m_pctlPhotoTheme->Append(strThemes);
	
	//////////////////////////////////////////////////////////////////////////
	// Make the initial selections
	//////////////////////////////////////////////////////////////////////////

	if (strThemes.Count() >= 1)
		m_pctlThemeList->SetSelection(0);
		
	m_pctlNightTheme->SetStringSelection( m_nightTheme );
	m_pctlDayTheme->SetStringSelection( m_dayTheme );
	m_pctlPhotoTheme->SetStringSelection( m_photoTheme );
	m_pctlThemeUsage->SetSelection( m_themeUsage );
}

// save items
void mapThemeOption::OnWindowDestroy()
{
    m_nightTheme = m_pctlNightTheme->GetStringSelection();
    m_dayTheme = m_pctlDayTheme->GetStringSelection();
    m_photoTheme = m_pctlPhotoTheme->GetStringSelection();
    m_themeUsage = (ThemeManager::ThemeUsage)m_pctlThemeUsage->GetSelection();
}


//////////////////////////////////////////////////////////////////////////////
///
/// Ok was pressed .. save the settings
///
//////////////////////////////////////////////////////////////////////////////
bool mapThemeOption::DoSave()
{
	m_ThemeManager.SetNightTheme( m_nightTheme );
	m_ThemeManager.SetDayTheme( m_dayTheme );
	m_ThemeManager.SetPhotoTheme( m_photoTheme );

	m_ThemeManager.SetThemeUsage( (ThemeManager::ThemeUsage)m_pctlThemeUsage->GetSelection() );

	// apply changes
	*m_pParentThemeManager = m_ThemeManager;

    return true;
}

//////////////////////////////////////////////////////////////////////////////
///
/// Add was pressed .. create a new theme and edit it
///
//////////////////////////////////////////////////////////////////////////////
void mapThemeOption::OnAdd(wxCommandEvent& event)
{
    mapThemeOption * _this = (mapThemeOption*)wxDynamicCast ( event.m_callbackUserData, wxOptionPtr )->option;

	// Don't store the theme in the theme manager until
	// after it has been edited. Give the user an opportunity
	// to change it's name.
	//
	// After edited, need to update combo boxes
	int iIndex;
	MapAppearanceSettings cNewTheme = _this->m_ThemeManager.GetTheme(wxT("Default-Day"));
	wxString strNewTheme = wxString::Format(wxT("Theme%d"), _this->m_ThemeManager.GetThemeCount() + 1);

	if (_this->EditTheme(strNewTheme, cNewTheme) != wxID_OK)
		return;

	_this->m_ThemeManager.AddTheme(strNewTheme, cNewTheme);

	_this->RefreshComboBoxes();

	// make the added theme the selected theme
	iIndex = _this->m_pctlThemeList->FindString(strNewTheme);
	wxASSERT(iIndex >= 0);
	_this->m_pctlThemeList->SetSelection(iIndex);
}

//////////////////////////////////////////////////////////////////////////////
///
/// Edit was pressed .. edit the selected theme
///
//////////////////////////////////////////////////////////////////////////////
void mapThemeOption::OnEdit(wxCommandEvent& event)
{
    mapThemeOption * _this = (mapThemeOption*)wxDynamicCast ( event.m_callbackUserData, wxOptionPtr )->option;

	wxString strOldThemeName = _this->m_pctlThemeList->GetStringSelection();
	wxString strNewThemeName = strOldThemeName;
	
	if (_this->EditTheme(strNewThemeName, _this->m_ThemeManager.GetTheme(strOldThemeName)) != wxID_OK)
		return;

	if (strNewThemeName != strOldThemeName)
	{
		// the theme was renamed
		_this->m_ThemeManager.RenameTheme(strOldThemeName, strNewThemeName);
		_this->RefreshComboBoxes(strOldThemeName, strNewThemeName);
	}
}

//////////////////////////////////////////////////////////////////////////////
///
/// Remove was pressed .. remove the selected theme, provided it is not the
/// last one.
///
//////////////////////////////////////////////////////////////////////////////
void mapThemeOption::OnRemove(wxCommandEvent& event)
{
    mapThemeOption * _this = (mapThemeOption*)wxDynamicCast ( event.m_callbackUserData, wxOptionPtr )->option;

	wxString strTheme = _this->m_pctlThemeList->GetStringSelection();

	if (_this->m_pctlThemeList->GetCount() < 2)
	{
		wxMessageBox(wxT("You cannot delete the last theme."), wxT("Error"), wxICON_ERROR | wxOK);
		return;
	}

	if (wxMessageBox(	wxString::Format(wxT("Are you sure you want to permanently delete %s?"), wx_c_str(strTheme)), 
						wxT("Confirm Deletion"), 
						wxYES_NO | wxICON_QUESTION) != wxYES)
		return;

	_this->m_ThemeManager.RemoveTheme(strTheme);
	_this->RefreshComboBoxes();
}

//////////////////////////////////////////////////////////////////////////////
///
/// Edit the theme
///
//////////////////////////////////////////////////////////////////////////////
int mapThemeOption::EditTheme(wxString& strTheme, MapAppearanceSettings & theme)
{
	int iRtn;
	EditThemeDialog dlg(NULL, strTheme, theme, m_pMapData);
	
	iRtn = dlg.ShowModal();
	
	if (iRtn == wxID_OK)
	{
		theme = dlg.GetTheme();
		strTheme = dlg.GetThemeName();
	}
	
	return iRtn;
}

//////////////////////////////////////////////////////////////////////////////
///
/// The list of theme names has changed... refresh all the combo boxes
///
//////////////////////////////////////////////////////////////////////////////
void mapThemeOption::RefreshComboBoxes(const wxString & strRenameFrom, const wxString & strRenameTo)
{
	// Get current selection
	wxString strNightTheme		= m_pctlNightTheme->GetStringSelection();
	wxString strDayTheme		= m_pctlDayTheme->GetStringSelection();
	wxString strPhotoTheme		= m_pctlPhotoTheme->GetStringSelection();
	wxString strSelectedTheme	= m_pctlThemeList->GetStringSelection();
	int iSel;
	
	if (strNightTheme == strRenameFrom)
		strNightTheme = strRenameTo;
	if (strDayTheme == strRenameFrom)
		strDayTheme = strRenameTo;
	if (strPhotoTheme == strRenameFrom)
		strPhotoTheme = strRenameTo;
	if (strSelectedTheme == strRenameFrom)
		strSelectedTheme = strRenameTo;

	m_pctlThemeList->Clear();
	m_pctlNightTheme->Clear();
	m_pctlDayTheme->Clear();
	m_pctlPhotoTheme->Clear();

	wxArrayString strThemes = m_ThemeManager.GetThemeNames();

	m_pctlThemeList->Append(strThemes);
	m_pctlNightTheme->Append(strThemes);
	m_pctlDayTheme->Append(strThemes);
	m_pctlPhotoTheme->Append(strThemes);
	
	if (strThemes.GetCount() == 0)
		return;

	// Restore current selection
	iSel = m_pctlThemeList->FindString(strSelectedTheme);
	if (iSel >= 0)
		m_pctlThemeList->SetSelection(iSel);
	else
		m_pctlThemeList->SetSelection(0);
		
	iSel = m_pctlNightTheme->FindString(strNightTheme);
	if (iSel >= 0)
		m_pctlNightTheme->SetSelection(iSel);
	else
		m_pctlNightTheme->SetSelection(0);
		
	iSel = m_pctlDayTheme->FindString(strDayTheme);
	if (iSel >= 0)
		m_pctlDayTheme->SetSelection(iSel);
	else
		m_pctlDayTheme->SetSelection(0);

	iSel = m_pctlPhotoTheme->FindString(strPhotoTheme);
	if (iSel >= 0)
		m_pctlPhotoTheme->SetSelection(iSel);
	else
		m_pctlPhotoTheme->SetSelection(0);
}
