/*
 *  Roadnav
 *
 *  PreferencesPageThemes.h
 *
 *  Copyright (c) 2004 - 2008 Richard L. Lynch <rllynch@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of version 2 of the GNU General Public License
 *  as published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

//////////////////////////////////////////////////////////////////////////////
/// \file
///
/// Contains the themes options
///
//////////////////////////////////////////////////////////////////////////////

#ifndef OPTIONS_THEMES_H
#define OPTIONS_THEMES_H

#include "ThemeManager.h"
#include "libroadnav/MapData.h"

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Implements the map themes option
///
//////////////////////////////////////////////////////////////////////////////
class mapThemeOption : public wxEvtHandler, public wxOption {
	public:
	    mapThemeOption(IMapData * pMapData, ThemeManager * pThemeManager);
        
		void DoDisplay(wxOptionSizer * sizer, wxWindow * parent);
		
		void OnWindowDestroy();
		bool DoSave();
        bool Enable(bool enable){ return true; }
	
	private:
		void OnAdd(wxCommandEvent& event);
		void OnEdit(wxCommandEvent& event);
		void OnRemove(wxCommandEvent& event);
		
		int EditTheme(wxString& strTheme, MapAppearanceSettings & theme);
		void RefreshComboBoxes(const wxString & strRenameFrom = wxT(""), const wxString & strRenameTo = wxT(""));

		/// List of all themes	
		wxListBox* m_pctlThemeList;
		
		/// Theme used at night
		wxComboBox* m_pctlNightTheme;
        wxString m_nightTheme;
		
		/// Theme used during the day
		wxComboBox* m_pctlDayTheme;
        wxString m_dayTheme;
		
		/// Theme used in aerial photo mode
		wxComboBox* m_pctlPhotoTheme;
        wxString m_photoTheme;
		
		/// Always day/always night/switch at sunrise and sunset
		wxRadioBox* m_pctlThemeUsage;
        ThemeManager::ThemeUsage m_themeUsage;

		/// Map control data
		IMapData * m_pMapData;
		
		/// Parent's theme manager
		ThemeManager * m_pParentThemeManager;
		
		/// Currently being edited theme manager
		ThemeManager m_ThemeManager;
};

#endif
