/*
 *  Roadnav
 *  PreferencesPageLCDproc.cpp
 *
 *  Copyright (c) 2004 - 2008 Richard L. Lynch <rllynch@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of version 2 of the GNU General Public License
 *  as published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

//////////////////////////////////////////////////////////////////////////////
/// \file
///
/// Contains the LCDproc options
///
//////////////////////////////////////////////////////////////////////////////

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

/*
#ifdef _MSC_VER
#pragma warning(disable: 4786)
#pragma warning(disable: 4800)
#endif
*/

#include <wx/wx.h>

#include <libroadnav/Debug.h>

#include "wxOptions.h"
#include "OptionsLCDproc.h"

using namespace std;


//////////////////////////////////////////////////////////////////////////////
///
/// \brief Constructor -- Get initial options
///
//////////////////////////////////////////////////////////////////////////////
LCDprocOption::LCDprocOption() : wxOption(wxT(""), wxT(""))
{
	m_cActiveSettings = GetLCDprocSettings();
}
		

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Create and initialize controls
///
//////////////////////////////////////////////////////////////////////////////

void LCDprocOption::DoDisplay(wxOptionSizer * sizer, wxWindow * parent)
{
	wxStaticBoxSizer * psizerBottom;
	wxFlexGridSizer * psizerWidgetList;
	wxFlexGridSizer * psizerWidgetSettings;
	wxFlexGridSizer * psizerRowCol;
	wxBoxSizer * psizerWidgetListActions;
	wxString strTmp;
	unsigned int iVariable;
	
	wxArrayString * variables;
	
	variables = GetLCDprocVariableNames();

	//////////////////////////////////////////////////////////////////////////
	// "Enable LCDproc Support" label
	//////////////////////////////////////////////////////////////////////////
	sizer->Add(
		new wxStaticText(parent, wxID_ANY, wxT("Enable LCDproc Support")),
		0,
		wxALIGN_CENTER_VERTICAL | wxALIGN_RIGHT,
		0
	);
	
	//////////////////////////////////////////////////////////////////////////
	// "Enable LCDproc Support" check box
	//////////////////////////////////////////////////////////////////////////

	m_pctlEnableLCDproc = new wxCheckBox(parent, wxID_ANY, wxT(""));
	m_pctlEnableLCDproc->SetValue(m_cActiveSettings.m_bLCDprocEnabled);

	sizer->Add(
		m_pctlEnableLCDproc,
		0,
		wxALIGN_LEFT,
		0
	);

	
	//////////////////////////////////////////////////////////////////////////
	// "Host and Port" label
	//////////////////////////////////////////////////////////////////////////
	sizer->Add(
		new wxStaticText(parent, wxID_ANY, wxT("Host and Port")),
		0,
		wxALIGN_CENTER_VERTICAL | wxALIGN_RIGHT,
		0
	);
	
	//////////////////////////////////////////////////////////////////////////
	// "Host and Port" text control
	//////////////////////////////////////////////////////////////////////////

	m_pctlHostAndPort = new wxTextCtrl(parent, wxID_ANY, m_cActiveSettings.m_strHostAndPort, wxDefaultPosition, wxSize(250, -1));

	sizer->Add(
		m_pctlHostAndPort, 
		1,
		wxALIGN_LEFT,
		0
	);

	

	//////////////////////////////////////////////////////////////////////////
	// "Screen Selection" label
	//////////////////////////////////////////////////////////////////////////
	sizer->Add(
		new wxStaticText(parent, wxID_ANY, wxT("Screen Selection")),
		1,
		wxALIGN_CENTER_VERTICAL | wxALIGN_RIGHT,
		0
	);
	
	//////////////////////////////////////////////////////////////////////////
	// "Screen Selection" check box
	//////////////////////////////////////////////////////////////////////////

	m_pctlScreenSelection = new wxComboBox(parent, wxID_ANY, wxT(""), wxDefaultPosition, wxSize(250, -1), 0, NULL, wxCB_DROPDOWN);

	sizer->Add(
		m_pctlScreenSelection, 
		1,
		wxALIGN_LEFT,
		0
	);

	//////////////////////////////////////////////////////////////////////////
	// Bottom half of the screen
	//////////////////////////////////////////////////////////////////////////

	psizerBottom = new wxStaticBoxSizer(wxHORIZONTAL, parent, wxT("Screen Settings"));
	
	psizerWidgetSettings = new wxFlexGridSizer(1, 0, 10);

	//////////////////////////////////////////////////////////////////////////
	// "Type" label
	//////////////////////////////////////////////////////////////////////////
	psizerWidgetSettings->Add(
		new wxStaticText(parent, wxID_ANY, wxT("Widget Type")),
		1,
		wxALIGN_CENTER,
		0
	);
	
	//////////////////////////////////////////////////////////////////////////
	// "Type" combo box
	//////////////////////////////////////////////////////////////////////////

	m_pctlWidgetType = new wxComboBox(parent, wxID_ANY, wxT(""), wxDefaultPosition, wxSize(150, -1), 0, NULL, wxCB_READONLY);

	m_pctlWidgetType->Append(wxT("Text"));
	m_pctlWidgetType->Append(wxT("Title"));
	m_pctlWidgetType->Append(wxT("Horizontal Bar"));
	m_pctlWidgetType->Append(wxT("Vertical Bar"));
	
	m_pctlWidgetType->SetValue(wxT("Text"));

	psizerWidgetSettings->Add(
		m_pctlWidgetType, 
		1,
		wxALIGN_LEFT,
		0
	);

	
	
	//////////////////////////////////////////////////////////////////////////
	// Row/Column selection
	//////////////////////////////////////////////////////////////////////////

	psizerRowCol = new wxFlexGridSizer(2, 2, 10);

	//////////////////////////////////////////////////////////////////////////
	// "Row" label
	//////////////////////////////////////////////////////////////////////////
	psizerRowCol->Add(
		new wxStaticText(parent, wxID_ANY, wxT("Row")),
		1,
		wxALIGN_CENTER,
		0
	);
	
	//////////////////////////////////////////////////////////////////////////
	// "Column" label
	//////////////////////////////////////////////////////////////////////////
	psizerRowCol->Add(
		new wxStaticText(parent, wxID_ANY, wxT("Column")),
		1,
		wxALIGN_CENTER,
		0
	);
	
	//////////////////////////////////////////////////////////////////////////
	// "Row" text control
	//////////////////////////////////////////////////////////////////////////
	m_pctlWidgetRow = new wxTextCtrl(parent, wxID_ANY, wxT(""), wxDefaultPosition, wxSize(75, -1));

	psizerRowCol->Add(
		m_pctlWidgetRow, 
		1,
		wxALIGN_LEFT | wxGROW,
		0
	);

	//////////////////////////////////////////////////////////////////////////
	// "Column" text control
	//////////////////////////////////////////////////////////////////////////
	m_pctlWidgetColumn = new wxTextCtrl(parent, wxID_ANY, wxT(""), wxDefaultPosition, wxSize(75, -1));

	psizerRowCol->Add(
		m_pctlWidgetColumn, 
		1,
		wxALIGN_LEFT | wxGROW,
		0
	);

	psizerWidgetSettings->Add(psizerRowCol, 1, wxALL | wxGROW, 0);


	//////////////////////////////////////////////////////////////////////////
	// "Contents" label
	//////////////////////////////////////////////////////////////////////////
	psizerWidgetSettings->Add(
		new wxStaticText(parent, wxID_ANY, wxT("Widget Contents")),
		1,
		wxALIGN_CENTER,
		0
	);
	
	//////////////////////////////////////////////////////////////////////////
	// "Contents" combo box
	//////////////////////////////////////////////////////////////////////////

	m_pctlWidgetContents = new wxComboBox(parent, wxID_ANY, wxT(""), wxDefaultPosition, wxSize(150, -1), 0, NULL, 0);
	m_pctlWidgetContents->Connect(m_pctlWidgetContents->GetId(), wxEVT_COMMAND_COMBOBOX_SELECTED, 
		wxCommandEventHandler(LCDprocOption::OnScreenSelection), wxDynamicCast(new wxOptionPtr(this),wxObject));
	m_pctlWidgetContents->Connect(m_pctlWidgetContents->GetId(), wxEVT_COMMAND_TEXT_UPDATED, 
		wxCommandEventHandler(LCDprocOption::OnScreenSelection), wxDynamicCast(new wxOptionPtr(this),wxObject));

	psizerWidgetSettings->Add(
		m_pctlWidgetContents, 
		1,
		wxALIGN_LEFT,
		0
	);

	for (iVariable = 0; iVariable < variables->Count(); iVariable++)
		m_pctlWidgetContents->Append((*variables)[iVariable]);
	
	
	
	psizerWidgetList = new wxFlexGridSizer(2, 2, 10);
	
	//////////////////////////////////////////////////////////////////////////
	// "Widgets" label
	//////////////////////////////////////////////////////////////////////////
	psizerWidgetList->Add(
		new wxStaticText(parent, -1, wxT("Widget List")),
		1,
		wxALIGN_CENTER,
		0
	);
	
	//////////////////////////////////////////////////////////////////////////
	// Blank label
	//////////////////////////////////////////////////////////////////////////
	psizerWidgetList->Add(
		new wxStaticText(parent, -1, wxT("")),
		1,
		wxALIGN_CENTER,
		0
	);
	
	//////////////////////////////////////////////////////////////////////////
	// "Widgets" list box
	//////////////////////////////////////////////////////////////////////////

	m_pctlWidgetList = new wxListBox(parent, wxID_ANY, wxDefaultPosition, wxSize(100, 100), 0, NULL, wxLB_SINGLE);
	m_pctlWidgetList->Connect(m_pctlWidgetList->GetId(), wxEVT_COMMAND_LISTBOX_SELECTED, 
		wxCommandEventHandler(LCDprocOption::OnWidgetSelection), wxDynamicCast(new wxOptionPtr(this),wxObject));

	psizerWidgetList->Add(
		m_pctlWidgetList, 
		1,
		wxALIGN_CENTER | wxGROW,
		0
	);

	
	//////////////////////////////////////////////////////////////////////////
	// Widget list actions - add, update, delete
	//////////////////////////////////////////////////////////////////////////
	psizerWidgetListActions = new wxBoxSizer(wxVERTICAL);

	wxButton * tmp;
	
	tmp = new wxButton(parent, wxID_ANY, wxT("&Add"));
	tmp->Connect(tmp->GetId(), wxEVT_COMMAND_BUTTON_CLICKED, 
		wxCommandEventHandler(LCDprocOption::OnWidgetAdd), wxDynamicCast(new wxOptionPtr(this),wxObject));

	psizerWidgetListActions->Add(
		tmp, 
		1, 
		wxALL | wxGROW, 
		0
	);

	tmp = new wxButton(parent, wxID_ANY, wxT("&Update"));
	tmp->Connect(tmp->GetId(), wxEVT_COMMAND_BUTTON_CLICKED, 
		wxCommandEventHandler(LCDprocOption::OnWidgetUpdate), wxDynamicCast(new wxOptionPtr(this),wxObject));

	psizerWidgetListActions->Add(
		tmp, 
		1, 
		wxALL | wxGROW, 
		0
	);


	tmp = new wxButton(parent, wxID_ANY, wxT("&Delete"));
	tmp->Connect(tmp->GetId(), wxEVT_COMMAND_BUTTON_CLICKED, 
		wxCommandEventHandler(LCDprocOption::OnWidgetDelete), wxDynamicCast(new wxOptionPtr(this),wxObject));

	psizerWidgetListActions->Add(
		tmp, 
		1, 
		wxALL | wxGROW, 
		0
	);

	psizerWidgetList->Add(psizerWidgetListActions, 1, wxALL | wxGROW, 0);

	//////////////////////////////////////////////////////////////////////////
	// Combine the sizers
	//////////////////////////////////////////////////////////////////////////
	psizerBottom->Add(psizerWidgetSettings, 1, wxALL | wxGROW, 10);
	psizerBottom->Add(psizerWidgetList, 1, wxALL | wxGROW, 10);
	
	wxOptionSizerFlags flags;
	flags.Expand();
	flags.ColSpan(2);

	sizer->Insert(sizer->GetChildren().size(), new wxOptionSizerItem(psizerBottom, flags));
	
	UpdateScreenList();
	
	if (m_pctlScreenSelection->GetCount() > 0)
	{
		m_pctlScreenSelection->SetSelection(0);
		UpdateWidgetList();
	}
}

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Keep the settings around for later
///
//////////////////////////////////////////////////////////////////////////////
void LCDprocOption::OnWindowDestroy()
{
	m_cActiveSettings.m_strHostAndPort = m_pctlHostAndPort->GetValue();
	m_cActiveSettings.m_bLCDprocEnabled = m_pctlEnableLCDproc->GetValue();
}

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Save the settings
///
//////////////////////////////////////////////////////////////////////////////
bool LCDprocOption::DoSave()
{
	SetLCDprocSettings(m_cActiveSettings);
	return true;
}


//////////////////////////////////////////////////////////////////////////////
///
/// \brief Update the list of screens in the m_pctlScreenSelection combo box.
///
//////////////////////////////////////////////////////////////////////////////
void LCDprocOption::UpdateScreenList()
{
	unsigned int iScreen;

	m_pctlScreenSelection->Clear();
	
	for (iScreen = 0; iScreen < m_cActiveSettings.m_vScreens.size(); iScreen++)
		m_pctlScreenSelection->Append(m_cActiveSettings.m_vScreens[iScreen].m_strName);
}

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Update the list of widgets in the m_pctlWidgetList list box.
///
//////////////////////////////////////////////////////////////////////////////
void LCDprocOption::UpdateWidgetList()
{
	int iScreen;
	
	iScreen = m_pctlScreenSelection->FindString(m_pctlScreenSelection->GetValue());
	
	m_pctlWidgetList->Clear();
	
	if (iScreen != wxNOT_FOUND && iScreen >= 0)
	{
		unsigned int iWidget;
		
		for (iWidget = 0; iWidget < m_cActiveSettings.m_vScreens[iScreen].m_vWidgets.size(); iWidget++)
			m_pctlWidgetList->Append(m_cActiveSettings.m_vScreens[iScreen].m_vWidgets[iWidget].m_strValue);
		
	}
}

/*
	Connect() doesn't keep the this pointer set correctly, so all of these
	contain a hack to ensure that the functions are being called on the 
	correct object. Nasty, but it works.
*/


//////////////////////////////////////////////////////////////////////////////
///
/// \brief Text was typed in the screen selection combo box - update the
/// widget list accordingly.
///
//////////////////////////////////////////////////////////////////////////////
void LCDprocOption::OnScreenSelection(wxCommandEvent& event)
{
	LCDprocOption * _this = (LCDprocOption*)wxDynamicCast ( event.m_callbackUserData, wxOptionPtr )->option;
	_this->UpdateWidgetList();
}

//////////////////////////////////////////////////////////////////////////////
///
/// \brief The user selected a widget from the list. Update the widget
/// settings on the screen with the settings for that widget.
///
//////////////////////////////////////////////////////////////////////////////
void LCDprocOption::OnWidgetSelection(wxCommandEvent& event)
{
	int iScreen;
	int iWidget;
	LCDprocOption * _this = (LCDprocOption*)wxDynamicCast ( event.m_callbackUserData, wxOptionPtr )->option;
	
	iScreen = _this->m_pctlScreenSelection->FindString(_this->m_pctlScreenSelection->GetValue());
	iWidget = _this->m_pctlWidgetList->GetSelection();
	
	if (iWidget == wxNOT_FOUND || iWidget < 0 || iScreen == wxNOT_FOUND || iScreen < 0)
	{
		_this->m_pctlWidgetRow->SetValue(wxT(""));
		_this->m_pctlWidgetColumn->SetValue(wxT(""));
		_this->m_pctlWidgetType->SetValue(wxT("Text"));
		_this->m_pctlWidgetContents->SetValue(wxT(""));
		
		return;
	}
		
	_this->m_pctlWidgetRow->SetValue(wxString::Format(wxT("%d"), _this->m_cActiveSettings.m_vScreens[iScreen].m_vWidgets[iWidget].m_iRow));
	_this->m_pctlWidgetColumn->SetValue(wxString::Format(wxT("%d"), _this->m_cActiveSettings.m_vScreens[iScreen].m_vWidgets[iWidget].m_iCol));
	_this->m_pctlWidgetType->SetValue(_this->m_cActiveSettings.m_vScreens[iScreen].m_vWidgets[iWidget].m_strType);
	_this->m_pctlWidgetContents->SetValue(_this->m_cActiveSettings.m_vScreens[iScreen].m_vWidgets[iWidget].m_strValue);
}

//////////////////////////////////////////////////////////////////////////////
///
/// \brief User clicked add - add the entered settings into a new widget.
///
//////////////////////////////////////////////////////////////////////////////
void LCDprocOption::OnWidgetAdd(wxCommandEvent& event)
{
	LCDprocWidgetInfo cWidgetInfo;
	int iScreen;
	LCDprocOption * _this = (LCDprocOption*)wxDynamicCast ( event.m_callbackUserData, wxOptionPtr )->option;
	
	iScreen = _this->m_pctlScreenSelection->FindString(_this->m_pctlScreenSelection->GetValue());
	
	if (iScreen == wxNOT_FOUND || iScreen < 0)
	{
		LCDprocScreenInfo cScreenInfo;
		
		cScreenInfo.m_strName = _this->m_pctlScreenSelection->GetValue();
		_this->m_cActiveSettings.m_vScreens.push_back(cScreenInfo);

		iScreen = _this->m_cActiveSettings.m_vScreens.size() - 1;

		_this->UpdateScreenList();
		_this->m_pctlScreenSelection->SetSelection(iScreen);
	}
	
	cWidgetInfo.m_iCol = atoi(wx_c_str(_this->m_pctlWidgetColumn->GetValue()));
	cWidgetInfo.m_iRow = atoi(wx_c_str(_this->m_pctlWidgetRow->GetValue()));
	cWidgetInfo.m_strType = _this->m_pctlWidgetType->GetValue();
	cWidgetInfo.m_strValue = _this->m_pctlWidgetContents->GetValue();
	
	_this->m_cActiveSettings.m_vScreens[iScreen].m_vWidgets.push_back(cWidgetInfo);
	_this->UpdateWidgetList();
	_this->m_pctlWidgetList->SetSelection(_this->m_pctlWidgetList->GetCount() - 1);
}

//////////////////////////////////////////////////////////////////////////////
///
/// \brief User clicked update - update the settings in the selected widget
/// with what's on the screen.
///
//////////////////////////////////////////////////////////////////////////////
void LCDprocOption::OnWidgetUpdate(wxCommandEvent& event)
{
	LCDprocWidgetInfo cWidgetInfo;
	int iScreen;
	int iWidget;
	LCDprocOption * _this = (LCDprocOption*)wxDynamicCast ( event.m_callbackUserData, wxOptionPtr )->option;
	
	iScreen = _this->m_pctlScreenSelection->FindString(_this->m_pctlScreenSelection->GetValue());
	iWidget = _this->m_pctlWidgetList->GetSelection();
	
	if (iWidget == wxNOT_FOUND || iWidget < 0 || iScreen == wxNOT_FOUND || iScreen < 0)
		return;
	
	cWidgetInfo.m_iCol = atoi(wx_c_str(_this->m_pctlWidgetColumn->GetValue()));
	cWidgetInfo.m_iRow = atoi(wx_c_str(_this->m_pctlWidgetRow->GetValue()));
	cWidgetInfo.m_strType = _this->m_pctlWidgetType->GetValue();
	cWidgetInfo.m_strValue = _this->m_pctlWidgetContents->GetValue();
	
	_this->m_cActiveSettings.m_vScreens[iScreen].m_vWidgets[iWidget] = cWidgetInfo;
	
	_this->UpdateWidgetList();
	
	_this->m_pctlWidgetList->SetSelection(iWidget);
}

//////////////////////////////////////////////////////////////////////////////
///
/// \brief User clicked delete - delete the selected widget.
///
/// Also, deletes the selected screen if there are no widgets left on it.
///
//////////////////////////////////////////////////////////////////////////////
void LCDprocOption::OnWidgetDelete(wxCommandEvent& event)
{
	int iScreen;
	int iWidget;
	LCDprocOption * _this = (LCDprocOption*)wxDynamicCast ( event.m_callbackUserData, wxOptionPtr )->option;
	
	iScreen = _this->m_pctlScreenSelection->FindString(_this->m_pctlScreenSelection->GetValue());
	iWidget = _this->m_pctlWidgetList->GetSelection();
	
	if (iWidget == wxNOT_FOUND || iWidget < 0 || iScreen == wxNOT_FOUND || iScreen < 0)
		return;
	
	_this->m_cActiveSettings.m_vScreens[iScreen].m_vWidgets.erase(_this->m_cActiveSettings.m_vScreens[iScreen].m_vWidgets.begin() + iWidget);
	_this->UpdateWidgetList();

	// check if this was the last widget
	if (_this->m_cActiveSettings.m_vScreens[iScreen].m_vWidgets.size() == 0)
	{
		// if so, remove the entire screen
		_this->m_cActiveSettings.m_vScreens.erase(_this->m_cActiveSettings.m_vScreens.begin() + iScreen);
		_this->UpdateScreenList();
	}
}


