/*
 *  wxOptionGroup.cpp
 *
 *  Copyright (c) 2008 Dustin Spicuzza <dustin@virtualroadside.com>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of version 2 of the GNU General Public License
 *  as published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

//////////////////////////////////////////////////////////////////////////////
/// \file
///
/// Defines the implementation for wxOptionGroup.
///
/// This is part of a generic set of classes designed so that it can be used 
/// with any project, project-specific options should be added 
/// to OptionsLaunch.cpp
///
//////////////////////////////////////////////////////////////////////////////

#include <wx/wx.h>

#include "wxOptionGroup.h"
#include <wx/arrimpl.cpp>


WX_DEFINE_OBJARRAY(wxOptionGroups);
IMPLEMENT_DYNAMIC_CLASS(wxOptionGroupPtr, wxObject);


wxOptionGroup* wxOptionGroup::Add(wxOptionGroup * group)
{
	wxASSERT_MSG(m_options.Count() == 0, _T("You must either add groups, or add options, not both!!"));
	
	if (group){ 
		wxASSERT_MSG(group->m_parent == NULL, _T("param: wxOptionGroup instances should not be reused!"));
		m_optionGroups.Add(group);
		group->m_parent = this;
	}

	return this;
}

wxOptionGroup* wxOptionGroup::Add(wxOption * option)
{
	wxASSERT_MSG(m_optionGroups.Count() == 0, _T("You must either add options, or add groups, not both!!"));
	m_options.Add(option);
	option->SetParent(this);

	return this;
}

bool wxOptionGroup::Save()
{
	size_t i;
	bool ret = true;

	if (m_optionGroups.Count() > 0)
		for (i = 0; i < m_optionGroups.Count(); i++)
			ret = ret && m_optionGroups[i].Save();
	else
		for (i = 0; i < m_options.Count(); i++)
			m_options[i].option->Save();

	return ret;
}

